@@ -39,13 +39,14 @@ from PyInstaller.building.datastruct import TOC, Target, Tree, _check_guts_eq
 from PyInstaller.building.splash import Splash
 from PyInstaller.building.osx import BUNDLE
 from PyInstaller.building.toc_conversion import DependencyProcessor
-from PyInstaller.building.utils import \
-    _check_guts_toc_mtime, format_binaries_and_datas
+from PyInstaller.building.utils import _check_guts_toc_mtime, \
+    format_binaries_and_datas, _should_include_system_binary
 from PyInstaller.depend.utils import \
     create_py3_base_library, scan_code_for_ctypes
 from PyInstaller.archive import pyz_crypto
 from PyInstaller.utils.misc import \
     get_path_to_toplevel_modules, get_unicode_modules, mtime
+from PyInstaller.utils.hooks import exec_statement
 
 if is_win:
     from PyInstaller.utils.win32 import winmanifest
@@ -99,6 +100,47 @@ def setupUPXFlags():
     compat.setenv("UPX", f)
 
 
+def discover_hook_directories():
+    """
+    Discover hook directories via pkg_resources and pyinstaller40
+    entry points. Perform the discovery in a subprocess to avoid
+    importing the package(s) in the main process.
+
+    :return: list of discovered hook directories.
+    """
+
+    hook_directories = []
+    output = exec_statement("""
+        import sys
+        import pkg_resources
+
+        entry_points = pkg_resources.iter_entry_points(
+            'pyinstaller40', 'hook-dirs')
+        for entry_point in entry_points:
+            try:
+                hook_dirs = entry_point.load()()
+                for hook_dir in hook_dirs:
+                    print('\\n$_pyi:' + hook_dir + '*')
+            except Exception as e:
+                print("discover_hook_directories: Failed to process hook "
+                      "entry point '%s': %s" % (entry_point, e),
+                      file=sys.stderr)
+    """)
+
+    # use splitlines rather than split, because split can break on any white
+    # spaces in the path
+    for line in output.splitlines():
+        # Filter out extra output by checking for the special prefix
+        # and suffix
+        if line.startswith("$_pyi:") and line.endswith("*"):
+            hook_directories.append(line[6:-1])
+
+    logger.debug("discover_hook_directories: Hook directories: %s",
+                 hook_directories)
+
+    return hook_directories
+
+
 class Analysis(Target):
     """
     Class does analysis of the user's main Python scripts.
@@ -215,9 +257,7 @@ class Analysis(Target):
             self.hookspath.extend(hookspath)
 
         # Add hook directories from PyInstaller entry points.
-        for entry_point in pkg_resources.iter_entry_points(
-                'pyinstaller40', 'hook-dirs'):
-            self.hookspath += list(entry_point.load()())
+        self.hookspath += discover_hook_directories()
 
         self.hooksconfig = {}
         if hooksconfig:
@@ -450,8 +490,12 @@ class Analysis(Target):
         for name, co in ctypes_code_objs.items():
             # Get dlls that might be needed by ctypes.
             logger.debug('Scanning %s for shared libraries or dlls', name)
-            ctypes_binaries = scan_code_for_ctypes(co)
-            self.binaries.extend(set(ctypes_binaries))
+            try:
+                ctypes_binaries = scan_code_for_ctypes(co)
+                self.binaries.extend(set(ctypes_binaries))
+            except Exception as ex:
+                raise RuntimeError(f"Failed to scan the module '{name}'. "
+                                   f"This is a bug. Please report it.") from ex
 
         self.datas.extend(
             (dest, source, "DATA") for (dest, source) in
@@ -603,6 +647,19 @@ class Analysis(Target):
         binaries.append((os.path.basename(python_lib), python_lib, 'BINARY'))
         logger.info('Using Python library %s', python_lib)
 
+    def exclude_system_libraries(self, list_of_exceptions=[]):
+        """
+        This method may be optionally called from the spec file to exclude
+        any system libraries from the list of binaries other than those
+        containing the shell-style wildcards in list_of_exceptions.
+        Those that match '*python*' or are stored under 'lib-dynload' are
+        always treated as exceptions and not excluded.
+        """
+
+        self.binaries = \
+            [i for i in self.binaries
+                if _should_include_system_binary(i, list_of_exceptions)]
+
 
 class ExecutableBuilder(object):
     """
